@@ -1,4 +1,5 @@
 import pathlib
+import warnings
 
 import h5py
 import nrefocus
@@ -24,7 +25,7 @@ class QPImage(object):
     _instances = 0
 
     def __init__(self, data=None, bg_data=None, which_data="phase",
-                 meta_data={}, holo_kw={}, proc_phase=True,
+                 meta_data=None, holo_kw=None, proc_phase=True,
                  h5file=None, h5mode="a", h5dtype="float32"):
         """Quantitative phase image manipulation
 
@@ -38,7 +39,7 @@ class QPImage(object):
             The experimental data (see `which_data`)
         bg_data: 2d ndarray (float or complex), list, or `None`
             The background data (must be same type as `data`)
-        which_data: str
+        which_data: str or tuple
             String or comma-separated list of strings indicating
             the order and type of input data. Valid values are
             "hologram", "field", "phase", "phase,amplitude",
@@ -97,6 +98,10 @@ class QPImage(object):
             qpi = QPImage(data=...)
             qpi_scliced = qpi[10:20, 40:30]
         """
+        if holo_kw is None:
+            holo_kw = {}
+        if meta_data is None:
+            meta_data = {}
         if (data is not None and
                 not isinstance(data, (np.ndarray, list, tuple))):
             msg = "`data` must be numpy.ndarray!"
@@ -118,7 +123,7 @@ class QPImage(object):
             self.h5 = h5py.File(**h5kwargs)
             self._do_h5_cleanup = True
         QPImage._instances += 1
-        # set holo data
+        #: hologram processing keyword arguments
         self.holo_kw = holo_kw
         # set meta data
         meta = MetaDict(meta_data)
@@ -142,6 +147,8 @@ class QPImage(object):
             self.set_bg_data(bg_data=bg_data,
                              which_data=which_data)
         self.h5dtype = h5dtype
+        # :mod:`nrefocus` interface class
+        self._refocuser = None
 
     def __enter__(self):
         return self
@@ -200,7 +207,7 @@ class QPImage(object):
                                                )
         if "wavelength" in self:
             wl = self["wavelength"]
-            if wl < 2000e-9 and wl > 10e-9:
+            if 2000e-9 > wl > 10e-9:
                 # convenience for light microscopy
                 rep += ", λ={:.1f}nm".format(wl * 1e9)
             else:
@@ -274,9 +281,6 @@ class QPImage(object):
         amp, pha: tuple of (:class:`Amplitdue`, :class:`Phase`)
         """
         which_data = QPImage._conv_which_data(which_data)
-        if which_data not in VALID_INPUT_DATA:
-            msg = "`which_data` must be one of {}!".format(VALID_INPUT_DATA)
-            raise ValueError(msg)
 
         if which_data == "field":
             amp = np.abs(data)
@@ -293,6 +297,10 @@ class QPImage(object):
         elif which_data == "hologram":
             amp, pha = self._get_amp_pha(holo.get_field(data, **self.holo_kw),
                                          which_data="field")
+        else:
+            raise ValueError(
+                f"`which_data` must be one of {VALID_INPUT_DATA}!")
+
         if amp.size == 0 or pha.size == 0:
             msg = "`data` with shape {} has zero size!".format(amp.shape)
             raise ValueError(msg)
@@ -535,15 +543,23 @@ class QPImage(object):
         h5 = copyh5(self.h5, h5file)
         return QPImage(h5file=h5, h5dtype=self.h5dtype)
 
-    def refocus(self, distance, method="helmholtz", h5file=None, h5mode="a"):
+    def refocus(self, distance, kernel="helmholtz", padding=True,
+                h5file=None, h5mode="a", ret_refocus_iface=False,
+                method=None):
         """Compute a numerically refocused QPImage
 
         Parameters
         ----------
         distance: float
             Focusing distance [m]
-        method: str
+        kernel: str
             Refocusing method, one of ["helmholtz","fresnel"]
+        padding: bool
+            Whether or not to perform padding during refocusing.
+            You may disable padding if your input image does not
+            have any discontinuities at the border (i.e. you can
+            tile your input image and it would look good), otherwise
+            you will experience ringing artifacts.
         h5file: str, h5py.Group, h5py.File, or None
             A path to an hdf5 data file where the QPImage is cached.
             If set to `None` (default), all data will be handled in
@@ -561,6 +577,13 @@ class QPImage(object):
             - "w": Create file, truncate if exists
             - "w-" or "x": Create file, fail if exists
             - "a": Read/write if exists, create otherwise (default)
+        ret_refocus_iface: bool
+            Whether or not to also return the `nrefocus.Refocus*` class
+            used for refocusing; this might be useful if you wish to
+            do quick manual refocusing or have to create a refocusing
+            series.
+        method: str
+            deprecated, use `kernel` instead
 
         Returns
         -------
@@ -569,28 +592,50 @@ class QPImage(object):
 
         See Also
         --------
-        :mod:`nrefocus`: library used for numerical focusing
+        - :mod:`nrefocus`: library used for numerical focusing
+        - :ref:`nrefocus:sec_refocus_interface`: refocusing interface
         """
-        field2 = nrefocus.refocus(field=self.field,
-                                  d=distance/self["pixel size"],
-                                  nm=self["medium index"],
-                                  res=self["wavelength"]/self["pixel size"],
-                                  method=method
-                                  )
+        if method is not None:
+            warnings.warn("`method` is deprecated, use `kernel` instead!",
+                          DeprecationWarning)
+            if kernel is not None:
+                kernel = method
+        # `self.field` this is always computed anew, so we don't have to worry
+        # about comparing against the same object.
+        field = self.field
+        if (self._refocuser is None
+            or not np.all(self._refocuser.origin == field)
+            or self._refocuser.kernel != kernel
+                or self._refocuser.padding != padding):
+            # We have a new field, so we have to create a new Refocus
+            refocus_iface = nrefocus.get_best_interface()
+            self._refocuser = refocus_iface(
+                field=field,
+                wavelength=self["wavelength"],
+                pixel_size=self["pixel size"],
+                medium_index=self["medium index"],
+                distance=0,
+                kernel=kernel,
+                padding=padding
+            )
+        field2 = self._refocuser.propagate(distance)
+
         if "identifier" in self:
             ident = self["identifier"]
         else:
             ident = ""
         meta_data = self.meta
-        meta_data["identifier"] = "{}@{}{:.5e}m".format(ident,
-                                                        method[0],
-                                                        distance)
+        meta_data["identifier"] = f"{ident}@{kernel[0]}{distance:.5e}m"
+
         qpi2 = QPImage(data=field2,
                        which_data="field",
                        meta_data=meta_data,
                        h5file=h5file,
                        h5mode=h5mode)
-        return qpi2
+        if ret_refocus_iface:
+            return qpi2, self._refocuser
+        else:
+            return qpi2
 
     def set_bg_data(self, bg_data, which_data=None, proc_phase=True):
         """Set background amplitude and phase data
